
/*******************************************************************************
* Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
*
* This file is part of MassChroqPRM.
*
*     MassChroqPRM is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     MassChroqPRM is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with MassChroqPRM.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
******************************************************************************/

#ifndef PRMXIC_H
#define PRMXIC_H

#include <pappsomspp/msrun/msrunid.h>
#include <pappsomspp/peptide/peptide.h>
#include <pappsomspp/spectrum/spectrum.h>
#include <pappsomspp/peptide/peptidefragmentionlistbase.h>
#include <pappsomspp/xic/xicpeptidefragmention.h>
#include <pappsomspp/xic/xicpeptidefragmentionnaturalisotope.h>
#include <odsstream/calcwriterinterface.h>
#include <pappsomspp/xic/detection/xicdetectionzivy.h>

#include "../peptides/prmpeptide.h"
#include "../utils/masschroqprmexception.h"


using namespace pappso;

class PrmXic;

typedef std::shared_ptr<PrmXic> NoConstPrmXicSp;



class XicDetectionMaxSink: public XicDetectionSinkInterface {
public:
    void setXicPeak (XicPeak & xic_peak) override {
        _count++;
        if (xic_peak.getArea() > _peak_max.getArea()) {
            _peak_max = xic_peak;
        }
    };
    const XicPeak & getXicPeak() const {
        if (_count == 0) throw MassChroqPrmException(QObject::tr("no peak detected")) ;
        return _peak_max;
    };
private :
    unsigned int _count=0;
    XicPeak _peak_max;
};


class PrmXic
{
public:
    PrmXic(const MsRunId & msrun_id, const PrmPeptideSp & peptideSp, unsigned int z, PrecisionP precision);
    PrmXic (const PrmXic & other);
    ~PrmXic();

    virtual NoConstPrmXicSp makeNoConstPrmXicSp() const;
    pappso::mz getMz() const;
    virtual void push_back(const pappso::Spectrum & spectrum, pappso_double retentionTime);

    virtual void print(QTextStream & out) const;
    virtual void report(CalcWriterInterface & writer) const;

    virtual void detectXicTic();


    static void reportGeneralHeader(CalcWriterInterface & swriter);
    virtual void reportGeneralLine(CalcWriterInterface & writer) const;
protected :
    pappso_double computeTicArea() const;

protected :
    const MsRunId _msrun_id;
    const PrmPeptideSp _peptideSp;
    const unsigned int _z;
    
    
    PeptideFragmentIonListBaseSp _p_peptide_fragment_ion_list_sp;
    Xic _xic_tic;
    XicDetectionMaxSink _max_peak_tic;

    std::list<pappso::PeptideIon> _ionListToMonitor;

    PrecisionP _precision = Precision::getDaltonInstance(0.02);


private :
    void computeHyperscore(const pappso::Spectrum & spectrum);
    XicPeptideFragmentIon & findXic(const pappso::PeakIonMatch& peptide_ion_match);

    
private:
    std::vector<XicPeptideFragmentIon> _xicList;
    
    

};

#endif // PRMXIC_H
